#-*- coding: utf-8 -*-

#**********************************************
# oct2spec/Spec.py
#
# Class file for the class Spec
#
# Based on R2spec by Pierre-Yves chibon
#
# Copyright (C) 2011 Orion Poplawski <orion@cora.nwra.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#**********************************************

import os, sys
from jinja2 import Template

class Spec:
    """
    Spec Class
        Write the spec file.
    """
    
    def __init__(self, package = None):
        self.package = package
        self.dict = {}
        self.spec = ""    
        self.name = ""
        self.arch = "arch"
    
    def finishName(self, arch):
        ''' Generate the dictionnary use for the template
            and complete the information while not found '''
        
        if self.package.packager.name is None or self.package.packager.name == "":
            self.dict['name'] = 'Name'
        else:
            self.dict['name'] = self.package.packager.name
        
        if self.package.packager.email is None or self.package.packager.email == "":
            self.dict['email'] = 'Mail'
        else:
            self.dict['email'] = self.package.packager.email
        
        if self.package.source0 is None:
            self.package.source0 = ''
        else:
            self.dict['source0'] = self.package.source0
        
        if self.package.URL is None:
            self.package.URL = ''
        else:
            self.dict['URL'] = self.package.URL
            
        if self.package.description.summary is None or self.package.description.summary == '':
            print(" /!\ Summary empty /!\ ")
            self.dict['summary'] = "<      summary should be not be longer than this       >"
        else:
            self.dict['summary'] = self.package.description.summary + ' for Octave'
        
        if self.package.description.description is None or self.package.description.description =='':
            print(" /!\ Description empty /!\ ")
            self.dict['description'] = ''
        else:
            self.dict['description'] = self.package.description.description
        
        if arch:
            self.arch = arch
        self.dict['arch'] = self.arch
        
        # other options
        self.dict['version'] = self.package.description.version
        self.dict['octpkg'] = self.package.name
        self.dict['license'] = self.package.description.license 
        self.dict['date'] = self.package.date 
        self.dict['depends'] = self.package.description.depends
        self.dict['suggests'] = self.package.description.suggests
    
    def writeSpec(self, specname):
        ''' Write the spec file '''
                
        print("* Spec generated: \n  %s/%s" %(os.getcwd(), specname))
        try:
            tplfolder = os.path.dirname(__file__)
            f = open(tplfolder + '/specfile.tpl', 'r')
            tplfile = f.read()
            f.close()
            mytemplate = Template(tplfile)
            self.spec = mytemplate.render(self.dict)
        except IOError as err:
            print('Cannot read the file %s' %tplfolder + '/specfile.tpl')
            print(err)
            sys.exit(1)

        self.writeSpecTosystem(specname)


    def writeOut(self):
        ''' Print final advices '''
        print("*"*50)
        print("""TODO:
 * Check Summary
 * Check description
 * Check license
 * Add/Check Source0 and URL
 * Check Arch - We detected it as %s package but it needs to be checked, use rpmlint for it
 * Change Requires and BuildRequires
 * Change %%files
 """%self.arch)

    def readSpec(self, specname):
        ''' Read a given spec and keep it in memory '''
        print('* Read the file %s' %specname)
        self.name = specname
        try:
            f = open(self.name , 'r')
            self.spec = f.read()
            f.close()
        except IOError as err:
            print('Cannot read the file %s' %self.name)
            print(err)
        
    
    def cleanFiles(self):
        '''Remove all the content of the %files section'''
        print("* Clean the %file section")
        cnt = 0
        remove = False
        self.spec = self.spec.split("\n")
        while cnt < len(self.spec):
            row = self.spec[cnt]
            if row.startswith('%changelog'):
                remove = False
            if remove:
                del self.spec[cnt]
                cnt = cnt -1
            if row.startswith('%defattr'):
                remove = True
            cnt = cnt + 1
        
        self.spec = "\n".join(self.spec)
    
    def writeSpecTosystem(self, specname):
        ''' Write the spec in memory with the given name'''
        try:
            specfile = open(specname, 'w')
            specfile.write(self.spec)
            specfile.write("\n")
            specfile.close()
            print("* spec file written")
        except IOError as err:
            print('Cannot write the file %s' %specname)
            print(err)    

    def addFile(self, files):
        ''' Add to a spec file the given files list '''
        print("* Add files to the %file section")
        
        files.reverse()
        files = self.generateFiles(files)
        
        cnt = 0
        add = False
        self.spec = self.spec.split("\n")
        while cnt < len(self.spec):
            row = self.spec[cnt]
            if row.startswith('%changelog'):
                self.spec.insert(cnt, "")
                cnt = cnt + 1
                add = False
            if row.startswith('%defattr'):
                add = True
            if add:
                for file in files:
                    cnt = cnt + 1
                    self.spec.insert(cnt, file)
            cnt = cnt + 1
        
        self.spec = "\n".join(self.spec)

    def generateFiles(self, files):
        ''' Generate the list of files to add to the spec '''
        
        # TODO: deal with case where there no conventionnal file in the sources:
        # cf gdata
        macros = {
                '/usr/share/': '%{_datadir}/',
                '/usr/lib/': '%{_libdir}/',
                '/usr/lib64/': '%{_libdir}/',
                }

        doc = ['doc','html','description','news','copying','licence','citation']
        
        octpkg = self.name.split('.spec')[0].split('octave-')[1]
        
        linedocs = []
        linefiles = []
        for filename in files:
            folder, name = filename.split('R/library/%s/' %octpkg)
            name = name.strip().split('/')[0]
            if name.lower() in doc:
                linedocs.append("%doc " + "%%{rlibdir}/%s/"%octpkg  + name)
            else:
                linefiles.append("%%{rlibdir}/%s/"%octpkg  + name)
        
        fileout = []
        for doc in set(linedocs):
            fileout.append(doc)
        for file in set(linefiles):
            fileout.append(file)
        
        return fileout
                
